/* Copyright Yann Weber <asmsh@yannweb.net>
   This file is part of asmsh.

   asmsh is free software: you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation, either version 3 of the License, or any later version.
   
   asmsh is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
   details.
   
   You should have received a copy of the GNU General Public License along
   with asmsh. If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef ASMSH_SHELL_H
#define ASMSH_SHELL_H
#include "config.h"

#include <stdlib.h>
#include <unistd.h>

#include "asm_env.h"
#include "logger.h"
#include "compile.h"

typedef struct asmsh_s asmsh_t;
#include "shell_sym.h"

struct asmsh_s
{
	///! Compilation context
	asmsh_asmc_ctx_t *cctx;
	///! Child process environment
	asmsh_env_t *env;

	/** Stores the child process exec path
	 * @warning Can be NULL if a temporary file is used
	 * @deprecated
	 */
	char *child_path;
	/** May store the last executed instruction ? not sure if used or not */
	char *last_instr;
	/** Last executed bytecode ? */
	asmsh_bytecode_t last_bcode;

};

/** Initialize a shell environment
 * @param asmsh_t* Pointer on the shell environment
 * @param const char* The child process executable path (NULL is recommended
 * in order to use the embed executable)
 * @return 0 if no error else -1 and set errno
 */
int asmsh_init(asmsh_t *sh, const char *child_path);

/** Free all ressources holded by a shell environment
 * @param asmsh_t* Pointer on the shell environment
 */
void asmsh_cleanup(asmsh_t *sh);

/** 
 * @return <0 on error 0 on ok 1 on exit */
int asmsh_exec(asmsh_t *sh, const char *cmd);

/** Replace all labels found in given command by their relative offset
 * @param asmsh_t* The shell
 * @param char The prefix character before a symbol name
 * @param const char* The string we want to parse
 * @param char* A pointer on a buffer large enough to store the parsed string
 * @param size_t The buffer size
 * @return 0 on error else return the needed buffer size
 * @todo move in asm_env.*
 * @todo Enhance !!! Add support for relative or absolute address replacement
 */
size_t asmsh_parse_labels(asmsh_t *sh, char preffix, const char *cmd,
		char *buf, size_t buf_sz);

#include "shell_cmds.h" // declares the static commands list

#endif
